sub chosen_streams { 
    my ($sensor) = @_ ; 
    my %chosen =() ;
    my $streams = $sensor->samplerates ;
    foreach my $stream ( @$streams ) { 
	if ( $stream->samprate ne "" ) { 
	    $chosen{$stream->choice} = 1 ; 
	}
    }
    return %chosen ; 
}

sub all_chosen_streams { 
    my %chosen = () ;
    my $sensors = $Configuration->sensors ; 
    foreach my $sensor (@$sensors) { 
	%chosen = (%chosen, &chosen_streams($sensor)) ;
    }
    return %chosen ;
}

sub possible_streams {
    return @_ ; 
}

sub possible_streams_now {
    my %chosen = &all_chosen_streams() ;
    my @chosen = keys %chosen ;
    my @ok = &possible_streams(@chosen) ; 
    return @ok ; 
}

sub show_stream_choices { 
    my ($widget) = @_ ;
    my @ok = possible_streams_now() ;
    my $sensor = $Configuration->selected_sensor ; 
    my %chosen = chosen_streams($sensor) ;

    my @choices=() ; 
    foreach my $ok (@ok) { 
	if (! defined $chosen{$ok}) {
	    push(@choices, $ok) ; 
	}
    }
    unshift (@choices, "") ;
    $widget->delete(0,"end") ; 
    foreach ( @choices) { 
	$widget->insert("end", $_) ; 
    }
}

sub labelstreams {
    my ($frame, $row, @axes) = @_ ;
    my $emphaticColor = "#ffff00" ;
    my $Balloon = $frame->Balloon( -background=>$emphaticColor, -width=>60) ;   
    my $err = errno ; 

    my $col = 0 ; 
    my $e = $frame->Label  (
		-font=>"ptkformLabel",
		-width=>10,
		-text => "samprate",
		-fg=>"darkblue",
		)
	    ->grid(-row=>$row, -column=>$col++, -ipady=>0, -pady=>0, -sticky=>"nw" ) ;
	    $Balloon->attach($e, -balloonmsg=> "Choose a sample rate" ) ;

    foreach my $axis (@axes) { 
	$e = $frame->Label(
		    -text=>"  "
		    )
	    ->grid(-row=>$row, -column=>$col++, -ipady=>0, -pady=>0, -sticky=>"e" ) ;
	$e = $frame->Label(
		    -font=>"ptkformLabel",
		    -text => "on",
		    -fg=>"darkblue",
		    )
		->grid(-row=>$row, -column=>$col++, -ipady=>0, -pady=>0, -sticky=>"nw" ) ;
	$Balloon->attach($e, -balloonmsg=> "Enable this channel" ) ;

	$e = $frame->Label(
		    -text => "chan",
		    -font=>"ptkformLabel",
		    -width=>'3',
		    -fg=>"darkblue",
		    )
		->grid(-row=>$row, -column=>$col++, -ipady=>0, -pady=>0, -sticky=>"nw" ) ;
	$Balloon->attach($e, -balloonmsg=> "SEED channel name" ) ;

	$e = $frame->Label(
		    -text => "loc",
		    -font=>"ptkformLabel",
		    -width=>'2',
		    -fg=>"darkblue",
		    )
		->grid(-row=>$row, -column=>$col++, -ipady=>0, -pady=>0, -sticky=>"nw" ) ;
	$Balloon->attach($e, -balloonmsg=> "SEED loc code" ) ;

	$e = $frame->Label(
		    -text => "dlchan",
		    -font=>"ptkformLabel",
		    -width=>5,
		    -fg=>"darkblue",
		    )
		->grid(-row=>$row, -column=>$col++, -ipady=>0, -pady=>0, -sticky=>"nw" ) ;
	$Balloon->attach($e, -balloonmsg=> "Datalogger stream channel id" ) ;
    }
}

sub clearstream { 
    my ($frame, $row, $stream, @axes) = @_ ;
}

sub modified {
    my ($widget, $ref) = @_ ; 
    $ref->by_hand = 1 ; 
}

sub show_filter_stack {
    my ( $widget, $stream ) = @_ ;
    my $ok = "OK" ;
    my $b = $MW->Dialog(-title=>"Show Filter Stack",
                   -text=>"",
                   -bitmap => 'info',
                   -buttons=> [$ok] ) ;
    my $stages = $stream->stages ; 
    my $samprate = $stream->samprate ; 
    my $name = $stream->name ; 
    $b->configure( -text=>"$samprate hz $name* filter stack\n    " 
    	. join("\n    ", @$stages) ) ;
    $b->Show ;
}

sub add_empty_stream {
    my ($frame, $stream, $components, $row) = @_ ;
    my $col = 0 ;
    my $sensor = $Configuration->selected_sensor ; 
    my $samplerates = $sensor->samplerates ; 
    my $e = $frame->BrowseEntry  (
		-font=>"ptkformEntry",
		-listcmd => \&show_stream_choices,
		-browsecmd => [\&show_chosen_stream, $stream],
		-width=>10,
		-state=>"readonly",
		-variable=>\$stream->choice,
		-colorstate=>1, 
		)
	    ->grid(-row=>$row, -column=>$col++, -ipady=>0, -pady=>0, -sticky=>"nw" ) ;
    # $e->configure(-disabledforeground=>"black", -disabledbackground=>"white" ) ; 
    $Balloon->attach($e, -balloonmsg=> "Mouse-button to show filter stack" ) ;
    $e->bind("<ButtonRelease-1>", [\&show_filter_stack, $stream] ) ; 
    my $err = errno ; 
    my $channels = $stream->channels ; 
    foreach my $channel (@$channels) { 
	$e = $frame->Label(
		    -text=>"  "
		    )
	    ->grid(-row=>$row, -column=>$col++, -ipady=>0, -pady=>0, -sticky=>"e" ) ;
	$e->bind("<Key>", [\&modified, $channel] ) ; 
	$e = $frame->Checkbutton(
		    -variable=>\$channel->on,
		    -anchor=>"e",
		    -justify=>"right",
		    )
	    ->grid(-row=>$row, -column=>$col++, -ipady=>0, -pady=>0, -sticky=>"e" ) ;
	$err = errno ; 
	$e = $frame->Entry(
		    -textvariable=>\$channel->chan,
		    -font=>"ptkformEntry",
		    -width=>'3',
		    )
	    ->grid(-row=>$row, -column=>$col++, -ipady=>0, -pady=>0, -sticky=>"nw" ) ;
	$e->bind("<Key>", [\&modified, $channel] ) ; 
	$e = $frame->Entry(
		    -textvariable=>\$channel->loc,
		    -font=>"ptkformEntry",
		    -width=>'2',
		    )
	    ->grid(-row=>$row, -column=>$col++, -ipady=>0, -pady=>0, -sticky=>"nw" ) ;
	$e->bind("<Key>", [\&modified, $channel] ) ; 
	$e = $frame->Entry(
		    -textvariable=>\$channel->id,
		    -font=>"ptkformEntry",
		    -width=>5,
		    )
	    ->grid(-row=>$row, -column=>$col++, -ipady=>0, -pady=>0, -sticky=>"nw" ) ;
	$e->bind("<Key>", [\&modified, $channel] ) ; 
    }
}

1;
